'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.isPromise = undefined;
exports.default = reactTreeWalker;

var _react = require('react');

var defaultOptions = {
  componentWillUnmount: false

  // Lifted from https://github.com/sindresorhus/p-reduce
  // Thanks @sindresorhus!
}; /* eslint-disable no-console */

// Inspired by the awesome work done by the Apollo team.
// See https://github.com/apollostack/react-apollo/blob/master/src/server.ts
// This version has been adapted to be promise based.

// eslint-disable-next-line import/no-extraneous-dependencies
var pReduce = function pReduce(iterable, reducer, initVal) {
  return new Promise(function (resolve, reject) {
    var iterator = iterable[Symbol.iterator]();
    var i = 0;

    var next = function next(total) {
      var el = iterator.next();

      if (el.done) {
        resolve(total);
        return;
      }

      Promise.all([total, el.value]).then(function (value) {
        // eslint-disable-next-line no-plusplus
        next(reducer(value[0], value[1], i++));
      }).catch(reject);
    };

    next(initVal);
  });
};

// Lifted from https://github.com/sindresorhus/p-map-series
// Thanks @sindresorhus!
var pMapSeries = function pMapSeries(iterable, iterator) {
  var ret = [];

  return pReduce(iterable, function (a, b, i) {
    return Promise.resolve(iterator(b, i)).then(function (val) {
      ret.push(val);
    });
  }).then(function () {
    return ret;
  });
};

var ensureChild = function ensureChild(child) {
  return child && typeof child.render === 'function' ? ensureChild(child.render()) : child;
};

var isPromise = exports.isPromise = function isPromise(x) {
  return x != null && typeof x.then === 'function';
};

// Recurse an React Element tree, running visitor on each element.
// If visitor returns `false`, don't call the element's render function
// or recurse into its child elements
function reactTreeWalker(rootElement, visitor, rootContext) {
  var options = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : defaultOptions;

  return new Promise(function (resolve, reject) {
    var safeVisitor = visitor.isSafe ? visitor : function () {
      try {
        return visitor.apply(undefined, arguments);
      } catch (err) {
        reject(err);
      }
      return undefined;
    };

    var recursive = function recursive(currentElement, currentContext) {
      return new Promise(function (innerResolve) {
        var doVisit = function doVisit(getChildren, visitResult, childContext) {
          var visitChildren = function visitChildren() {
            var child = ensureChild(getChildren());
            var theChildContext = typeof childContext === 'function' ? childContext() : childContext;

            if (child == null) {
              // If no children then we can't traverse.  We've reached the leaf.
              innerResolve();
            } else if (_react.Children.count(child)) {
              // If its a react Children collection we need to breadth-first
              // traverse each of them.
              var mapper = function mapper(aChild) {
                return aChild ? recursive(aChild, theChildContext) : undefined;
              };
              var children = _react.Children.map(child, function (cur) {
                return cur;
              });
              // pMapSeries allows us to do depth-first traversal. Thanks @sindresorhus!
              pMapSeries(children, mapper).then(innerResolve, reject).catch(reject);
            } else {
              // Otherwise we pass the individual child to the next recursion.
              recursive(child, theChildContext).then(innerResolve, reject).catch(reject);
            }
          };

          if (visitResult === false) {
            // Visitor returned false, indicating a desire to not traverse.
            innerResolve();
          } else if (isPromise(visitResult)) {
            // We need to execute the result and pass it's result through to our
            // continuer.
            visitResult.then(function (promiseResult) {
              if (promiseResult === false) {
                innerResolve();
              } else {
                visitChildren();
              }
            }).catch(reject);
          } else {
            // Visitor returned true, indicating a desire to continue traversing.
            visitChildren();
          }
        };

        // Is this element a Component?
        if (typeof currentElement.type === 'function') {
          var Component = currentElement.type;
          var props = Object.assign({}, Component.defaultProps, currentElement.props);

          // Is this a class component? (http://bit.ly/2j9Ifk3)
          var isReactClassComponent = Component.prototype && (Component.prototype.isReactComponent || Component.prototype.isPureReactComponent);

          if (isReactClassComponent) {
            // React class component

            var instance = new Component(props, currentContext);

            // In case the user doesn't pass these to super in the constructor
            instance.props = instance.props || props;
            instance.context = instance.context || currentContext;

            // Make the setState synchronous.
            instance.setState = function (newState) {
              if (typeof newState === 'function') {
                // eslint-disable-next-line no-param-reassign
                newState = newState(instance.state, instance.props, instance.context);
              }
              instance.state = Object.assign({}, instance.state, newState);
            };

            doVisit(function () {
              // Call componentWillMount if it exists.
              if (instance.componentWillMount) {
                instance.componentWillMount();
              }

              var children = instance.render();

              if (options.componentWillUnmount && instance.componentWillUnmount) {
                try {
                  instance.componentWillUnmount();
                } catch (err) {
                  // This is an experimental feature, we don't want to break
                  // the bootstrapping process, but lets warn the user it
                  // occurred.
                  console.warn('Error calling componentWillUnmount whilst walking your react tree');
                  console.warn(err);
                }
              }

              return children;
            }, safeVisitor(currentElement, instance, currentContext), function () {
              return (
                // Ensure the child context is initialised if it is available. We will
                // need to pass it down the tree.
                instance.getChildContext ? Object.assign({}, currentContext, instance.getChildContext()) : currentContext
              );
            });
          } else {
            // Stateless Functional Component
            doVisit(function () {
              return Component(props, currentContext);
            }, safeVisitor(currentElement, null, currentContext), currentContext);
          }
        } else {
          // This must be a basic element, such as a string or dom node.
          doVisit(function () {
            return currentElement.props && currentElement.props.children ? currentElement.props.children : undefined;
          }, safeVisitor(currentElement, null, currentContext), currentContext);
        }
      });
    };

    recursive(rootElement, rootContext).then(resolve, reject);
  });
}